'use strict';

var archiver = require('archiver');
// https://github.com/archiverjs/node-archiver
var path = require('path');
var fs = require('fs');

/**
 * zipDir - (spromisowany) zipuje folder z możliwością zdefiniowania co zipujemy i katalogów, których nie chcemy załączać do archiwum.
 * @param  {str}    plikZip     nazwa pliku zip (z roszerzeniem .zip)
 * @param  {str}    coZipujemy  niewymagany parametr mozliwosc zipowania
 *                              relatywnego katalogu niż difoltowego cwd
 *                              (nie można zipować pojedyńczych plików)
 * @param  {arr}    coWylaczamy array z lista wyłączeń;
 *                              gdy null spakuje wszystko w katalogu.
 * @param  {bool}   gdyNieCWD   domyślnie false, czyli zip w CWD.
 *                              Natomiast gdy true, możemy zipa zapisać
 *                              gdziekolwiek na dysku, lecz parametr [plikZip]
 *                              musi posiadać pełną ścieżkę. Patrz przykłady.
 * @return I/O + promise        zip file + informacja z wilkością archiwum
 * @example
 * explorer.zipujKatalog("../test-wszystkot.zip", null, null, null);
 * explorer.zipujKatalog("../test-bezkropek.zip", null, ['dot'], null);
 * explorer.zipujKatalog("../test-bezkropekibezmodules.zip", null, ['dot', 'node_modules'], null);
 * explorer.zipujKatalog("../test-bezKat.zip", null, ['dot', 'node_modules', 'config', 'src', 'test'], null);
 * explorer.zipujKatalog("../test-bezKat.zip", null, ['dot', 'node_modules', 'config', 'src', 'test'], null)
  .then(function (msg) {})
  .catch(function (err){});
 * explorer.zipujKatalog("c:/jakisKat/test.zip", null, ['dot'], true);
 */
module.exports = function zipDir(plikZip, coZipujemy, coWylaczamy, gdyNieCWD) {
  var _whatToZip = path.join(coZipujemy || process.cwd());
  var _zipFile_name = plikZip || 'temp-zip';
  var _output;
  var _wylDot = true;
  var _wylFolders = ['**/*'];
  //['**/*','!node_modules{,/**}']
  if (coWylaczamy) {
    _wylDot = coWylaczamy.indexOf('dot') === -1 ? true : false;
    coWylaczamy.forEach(function (el) {
      if (el !== 'dot') _wylFolders.push('!' + el + '{,/**}');
    });
  }

  var _ifNoCWD = gdyNieCWD || false;

  var _archive = archiver('zip');

  if (_ifNoCWD) {
    _output = fs.createWriteStream(path.join(_zipFile_name));
  } else {
    _output = fs.createWriteStream(path.join(process.cwd(), _zipFile_name));
  }

  _archive.bulk([{ expand: true, cwd: _whatToZip, src: _wylFolders, dot: _wylDot }]);
  // console.log('_wylDot ' , _wylDot);
  // console.log('_wylFolders ' , _wylFolders);
  // console.log('_whatToZip ' , _whatToZip);

  // PROMISE
  return new Promise(function (resolve, reject) {
    _archive.on('error', function (err) {
      _archive.abort(); //not always useful but might save trouble
      reject('Unable to archive: ' + err);
      // throw err;
    });

    _archive.on('end', function () {
      // console.log('Archive finished!');
    });

    _archive.pipe(_output);
    _output.on('close', function () {
      resolve(_archive.pointer());
    });
    _archive.finalize();
  });
};